//
// CommonCcfConstituent -- XMM Science Analysis System
//
// Implementation of methods common to all concrete CcfConstituents
//
// Giuseppe Vacanti, XMM-SOC, European Space Agency
//   
// $Id: CommonCcfConstituent.cc,v 1.10 2001/12/21 14:26:01 gvacanti Exp $

// C++
#include <string>

// SAS
#include "Tracer.h"
#include "STime.h"
#include "DirEntry.h"
#include "DirEntryImplementation.h"
#include "Dal.h"
#include "errstr.h"

#include "CommonCcfConstituent.h"

void missingAttribute(string name, string attribute) {
  PROC
  errstr <<  "Missing " + attribute + " in " + name + ". Not a valid CCf constituent." << Sas::message(Msg::AppLib, Msg::Sparse);
  ENDPROC
}

CommonCcfConstituent::CommonCcfConstituent()
  : _initdate(STime("now")),
    _isValid(false),
    _init_done(false),
    _has_endvaldate(false)
  
{
  PROC
  ENDPROC
}

CommonCcfConstituent::CommonCcfConstituent(const string name)
  : DirEntryImplementation(name),
    _initdate(STime("now")),
    _isValid(true),
    _init_done(false),
    _has_endvaldate(false)

  
{
  PROC
  init();
  ENDPROC
}

void CommonCcfConstituent::print(ostream & os) const
{
  PROC
  os << "  name        = " << basename() << endl;
  os << "  directory   = " << dirname() << endl;
  os << "  type        = " << type() << endl;
  os << "  scope       = " << scope() << endl;
  os << "  issue       = " << issue() << endl;
  os << "  valdate     = " << valdate() << endl;
  if(hasEndvaldate())
    os << "  end valdate = " << endvaldate() << endl;
  else
    os << "  end valdate = none" << endl;
  os << "  subdate     = " << subdate() << endl;
  os << "  date        = " << date() << endl;
  os << "  size        = " << size() << endl;
  if(isSigned())
    os << "  signature   = " << signature() << endl;
  else
    os << "  signature   =  [not available]" << endl;
  os << "  category    = " << category() << endl;
    
  ENDPROC
}

void CommonCcfConstituent::init() {
  PROC
  
    // These are the symbolic names of the attributes we need to
    // identify in order to create the CIF.

  static const string TELESCOPE       = "TELESCOP";
  static const string SCOPE           = "SCOPE";
  static const string ISSUE           = "ISSUE";
  static const string TYPE_ID         = "TYPEID";
  static const string VALDATE         = "VALDATE";
  static const string EVALDATE        = "EVALDATE";
  // DATE is a reserved attribute and can be read through dataSetServer->date(set)
  // static const string DATE            = "DATE";
  static const string EXT_SEQUENCE    = "EXTSEQU";
  static const string EXT_SEQUENCE_ID = "EXTSEQID";
  static const string SUBDATE         = "SUBDATE";
  static const string CATEGORY        = "CATEGORY";
  // CREATOR is also a reserved attribute: dataSetServer->creator(set)
  // static const string CREATOR         = "CREATOR";

  static const string NAME            = "FNAME";
  static const string SIZE            = "FSIZE";

  DataSet * set = dataSetServer->open(name(),
				      DataSet::Read,
				      DataSetServer::Low);

  // Check that all the required attributes (as per CCF ICD) are there. 
  if(set->hasAttribute(TELESCOPE)) {
    _telescope = set->attribute(TELESCOPE)->asString();
  } else {
    _isValid = false;
    missingAttribute(TELESCOPE,basename());
  }

  if(set->hasAttribute(SCOPE)) {
    _scope = set->attribute(SCOPE)->asString();
  } else {
    _isValid = false;
    missingAttribute(SCOPE,basename());
  }

  if(set->hasAttribute(TYPE_ID)) {
    _type = set->attribute(TYPE_ID)->asString();
  } else {
    _isValid = false;
    missingAttribute(TYPE_ID,basename());
  }
  
  if(set->hasAttribute(VALDATE)) {
    _valdate = STime(set->attribute(VALDATE)->asString());
  } else {
    _isValid = false;
   missingAttribute(VALDATE,basename());
  }

  // End-Of-Validity date treated as optional
  if(set->hasAttribute(EVALDATE)) {
    _evaldate = STime(set->attribute(EVALDATE)->asString());
    _has_endvaldate = true;
  }

  if(set->hasAttribute(ISSUE)) {
    _issue = set->attribute(ISSUE)->asInt();
  } else {
    _isValid = false;
    missingAttribute(ISSUE,basename());
  }


  if(set->hasAttribute(EXT_SEQUENCE)) {
    _extSequence = set->attribute(EXT_SEQUENCE)->asString();
  } else {
    _isValid = false;
    missingAttribute(EXT_SEQUENCE,basename());
  }
  
  if(set->hasAttribute(EXT_SEQUENCE_ID)) {
    _extSequenceIdentifier = set->attribute(EXT_SEQUENCE_ID)->asString();
  } else {
    _isValid = false;
    missingAttribute(EXT_SEQUENCE_ID,basename());
  }

  if(set->hasAttribute(CATEGORY)){
    _category = set->attribute(CATEGORY)->asString();
  } else {
    _isValid = false;
    missingAttribute(CATEGORY,basename());
  }

  _date = STime(dataSetServer->date(set).c_str());

  if(set->hasAttribute(SUBDATE)) {
    _subdate = STime(set->attribute(SUBDATE)->asString());
  } else {
    _isValid = false;
    missingAttribute(SUBDATE,basename());
  }

  _creator = dataSetServer->creator(set);

  dataSetServer->close(set);

  _init_done = true;

  ENDPROC
}

bool CommonCcfConstituent::isValid() const {
  PROC
  RETURN _isValid;
}

// These methods return the data gathered by init.

string CommonCcfConstituent::telescope() const {
  PROC
  RETURN _telescope;
}

string CommonCcfConstituent::type() const {
  PROC
  RETURN _type;
}

string CommonCcfConstituent::scope() const {
  PROC
  RETURN _scope;
}

STime CommonCcfConstituent::valdate() const {
  PROC
  RETURN _valdate;
}

STime CommonCcfConstituent::endvaldate() const {
  PROC
  RETURN _evaldate;
}

bool CommonCcfConstituent::hasEndvaldate() const {
  PROC
  RETURN _has_endvaldate;
}

STime CommonCcfConstituent::date() const {
  PROC
  RETURN _date;
}

STime CommonCcfConstituent::subdate() const {
  PROC
  RETURN _subdate;
}

string CommonCcfConstituent::extSequence() const {
  PROC
  RETURN _extSequence;
}

string CommonCcfConstituent::extSequenceIdentifier() const {
  PROC
  RETURN _extSequenceIdentifier;
}

int CommonCcfConstituent::issue() const {
  PROC
  RETURN _issue;
}

string CommonCcfConstituent::creator() const {
  PROC
  RETURN _creator;
}

string CommonCcfConstituent::category() const {
  PROC
  RETURN _category;
}

bool CommonCcfConstituent::operator==(const CcfConstituent & rhs) const
{
  PROC
  RETURN rhs.scope() == scope() && rhs.type() == type() &&
    rhs.issue() == issue() && rhs.valdate() == valdate();
}

bool CommonCcfConstituent::operator!=(const CcfConstituent & rhs) const
{
  PROC
  RETURN ! (*this == rhs);
}

bool CommonCcfConstituent::operator<(const CcfConstituent & rhs) const
{
  PROC
    //    RETURN category() < rhs.category();

  if(type() == rhs.type() && scope() == rhs.scope())
    ALTRETURNV(issue() < rhs.issue())
  else {
    if(scope() == rhs.scope())
      ALTRETURNV(type() < rhs.type())
    else
      ALTRETURNV(scope() < rhs.scope())
  }
}

bool CommonCcfConstituent::operator>(const CcfConstituent & rhs) const
{
  PROC
    //    RETURN category() > rhs.category();

  if(type() == rhs.type() && scope() == rhs.scope())
    ALTRETURNV(issue() > rhs.issue())
  else {
    if(scope() == rhs.scope())
      ALTRETURNV(type() > rhs.type())
    else
      ALTRETURNV(scope() > rhs.scope())
  }
}

